@@ -5,7 +5,7 @@ use warnings;
 use 5.010;
 use parent qw/autobox/;
 
-our $VERSION = "1.022";
+our $VERSION = "1.023";
 
 =head1 NAME
 
@@ -27,9 +27,30 @@ particular when the values are hashrefs or objects.
 
 =head1 SYNOPSIS
 
-    use autobox::Core;  # uniq, sort, join, sum, etc.
+    use autobox::Core;  # map, uniq, sort, join, sum, etc.
     use autobox::Transform;
 
+=head2 Arrays
+
+    # use autobox::Core for ->map etc.
+
+    # filter (grep)
+    $book_locations->filter(); # true values
+    $books->filter(sub { $_->is_in_library($library) });
+
+    # Flatten arrayrefs-of-arrayrefs
+    $authors->map_by("books") # ->books returns an arrayref
+    # [ [ $book1, $book2 ], [ $book3 ] ]
+    $authors->map_by("books")->flat;
+    # [ $book1, $book2, $book3 ]
+
+    # Return reference, even in list context, e.g. in a parameter list
+    $book_locations->filter()->to_ref;
+
+    # Return array, even in scalar context
+    @books->to_array;
+
+
 =head2 Arrays with hashrefs/objects
 
     # $books and $authors below are arrayrefs with either objects or
@@ -38,10 +59,13 @@ particular when the values are hashrefs or objects.
     $books->map_by("genre");
     $books->map_by([ price_with_tax => $tax_pct ]);
 
+    $books->filter_by("is_sold_out");
+    $books->filter_by([ is_in_library => $library ]);
+    $books->filter_by([ price_with_tax => $rate ], sub { $_ > 56.00 });
+    $books->filter_by("price", sub { $_ > 56.00 });
+
+    # grep_by is an alias for filter_by
     $books->grep_by("is_sold_out");
-    $books->grep_by([ is_in_library => $library ]);
-    $books->grep_by([ price_with_tax => $rate ], sub { $_ > 56.00 });
-    $books->grep_by("price", sub { $_ > 56.00 });
 
     $books->uniq_by("id");
 
@@ -73,21 +97,6 @@ particular when the values are hashrefs or objects.
     # },
 
 
-=head2 Arrays
-
-    # Flatten arrayrefs-of-arrayrefs
-    $authors->map_by("books") # ->books returns an arrayref
-    # [ [ $book1, $book2 ], [ $book3 ] ]
-    $authors->map_by("books")->flat;
-    # [ $book1, $book2, $book3 ]
-
-    # Return reference, even in list context, e.g. in a parameter list
-    report( genres => $books->map_by("genre")->to_ref );
-
-    # Return array, even in scalar context
-    @books->to_array;
-
-
 =head2 Hashes
 
     # Upper-case the genre name, and make the count say "n books"
@@ -123,7 +132,7 @@ particular when the values are hashrefs or objects.
         ->map_by("name")->uniq->sort->join(", ");
 
     my $total_order_amount = $order->books
-        ->grep_by([ not_covered_by_vouchers => $vouchers ])
+        ->filter_by([ not_covered_by_vouchers => $vouchers ])
         ->map_by([ price_with_tax => $tax_pct ])
         ->sum;
 
@@ -138,11 +147,27 @@ and hashrefs.
 
 =item
 
+$array->filter()
+
+=item
+
+$array->flat()
+
+=item
+
+$array->as_ref()
+
+=item
+
+$array->as_array()
+
+=item
+
 $array->map_by()
 
 =item
 
-$array->grep_by()
+$array->filter_by()
 
 =item
 
@@ -160,18 +185,6 @@ $array->group_by_count()
 
 $array->group_by_array()
 
-=item
-
-$array->flat()
-
-=item
-
-$array->as_ref()
-
-=item
-
-$array->as_array()
-
 =back
 
 
@@ -251,8 +264,8 @@ sub _normalized_accessor_args_subref {
 
 =head2 Transforming lists of objects vs list of hashrefs
 
-C<map_by>, C<grep_by> etc. (all methods named C<*_by>) work with
-arrays that contain hashrefs or objects.
+C<map_by>, C<filter_by> etc. (all methods named C<*_by>) work with
+sets of hashrefs or objects.
 
 These methods are called the same way regardless of whether the array
 contains objects or hashrefs. The items in the list must all be either
@@ -266,21 +279,21 @@ item.
 
 =head3 Calling accessor methods with arguments
 
-Consider C<grep_by>:
+Consider C<filter_by>:
 
-    $array->grep_by($accessor, $subref)
+    $array->filter_by($accessor, $subref)
 
 If the $accessor is a string, it's a simple lookup/method call.
 
     # method call without args
-    $books->grep_by("price", sub { $_ < 15.0 })
+    $books->filter_by("price", sub { $_ < 15.0 })
     # becomes $_->price() or $_->{price}
 
 If the $accessor is an arrayref, the first item is the method name,
 and the rest of the items are the arguments to the method.
 
     # method call with args
-    $books->grep_by([ price_with_discount => 5.0 ], sub { $_ < 15.0 })
+    $books->filter_by([ price_with_discount => 5.0 ], sub { $_ < 15.0 })
     # becomes $_->price_with_discount(5.0)
 
 =head3 Deprecated syntax
@@ -289,8 +302,8 @@ There is an older syntax for calling methods with arguments. It was
 abandoned to open up more powerful ways to use grep/filter type
 methods. Here it is for reference, in case you run into existing code.
 
-    $array->grep_by($accessor, $args, $subref)
-    $books->grep_by("price_with_discount", [ 5.0 ], sub { $_ < 15.0 })
+    $array->filter_by($accessor, $args, $subref)
+    $books->filter_by("price_with_discount", [ 5.0 ], sub { $_ < 15.0 })
 
 Call the method $accessor on each object using the arguments in the
 $args arrayref like so:
@@ -330,27 +343,27 @@ context. E.g.
 
     $self->my_method(
         # Wrong, this is list context and wouldn't return an arrayref
-        books => $books->grep_by("is_published"),
+        books => $books->filter_by("is_published"),
     );
 
     $self->my_method(
         # Correct, convert the returned list to an arrayref
-        books => [ $books->grep_by("is_published") ],
+        books => [ $books->filter_by("is_published") ],
     );
     $self->my_method(
         # Correct, ensure scalar context to get an array ref
-        books => scalar $books->grep_by("is_published"),
+        books => scalar $books->filter_by("is_published"),
     );
 
     # Probably the nicest, since it goes at the end
     $self->my_method(
         # Correct, use ->to_ref to ensure an array reference is returned
-        books => $books->grep_by("is_published")->to_ref,
+        books => $books->filter_by("is_published")->to_ref,
     );
 
 
 
-=head1 AUTOBOX ARRAY METHODS
+=head1 METHODS ON ARRAY
 
 =cut
 
@@ -359,6 +372,119 @@ package # hide from PAUSE
 
 use autobox::Core;
 
+
+=head2 @array->filter($filter_subref = *is_true*) : @array | @$array
+
+Similar to Perl's C<grep>, return an @array with values for which
+$filter_subref returns a true value.
+
+The $filter_subref is called for each value to check whether this item
+should remain in the list (default is to check for true values).
+
+The $filter_subref should return a true value to remain. $_ is set to
+the current $value.
+
+Examples:
+
+    my @authors = $authors->filter(
+        sub { $_->name->publisher =~ /Orbit/ },
+    );
+
+
+=head3 filter and grep
+
+L<autobox::Core>'s C<grep> method takes a subref, just like this
+method.
+
+
+=cut
+
+sub filter {
+    my $array = shift;
+    my ($filter_subref) = @_;
+    $filter_subref //= sub { !! $_ };
+
+    my $result = eval {
+        [ CORE::grep { $filter_subref->( $_ ) } @$array ]
+    } or autobox::Transform::throw($@);
+
+    return wantarray ? @$result : $result;
+}
+
+=head2 @array->flat() : @array | @$array
+
+Return a (one level) flattened array, assuming the array items
+themselves are array refs. I.e.
+
+    [
+        [ 1, 2, 3 ],
+        [ "a", "b" ],
+        [ [ 1, 2 ], { 3 => 4 } ]
+    ]->flat
+
+returns
+
+    [ 1, 2, 3, "a", "b ", [ 1, 2 ], { 3 => 4 } ]
+
+This is useful if e.g. a C<-E<gt>map_by("some_method")> returns
+arrayrefs of objects which you want to do further method calls
+on. Example:
+
+    # ->books returns an arrayref of Book objects with a ->title
+    $authors->map_by("books")->flat->map_by("title")
+
+Note: This is different from autobox::Core's ->flatten, which reurns a
+list rather than an array and therefore can't be used in this
+way.
+
+=cut
+
+sub flat {
+    my $array = shift;
+    ###JPL: eval and report error from correct place
+    my $result = [ map { @$_ } @$array ];
+    return wantarray ? @$result : $result;
+}
+
+=head2 @array->to_ref() : $arrayref
+
+Return the reference to the @array, regardless of context.
+
+Useful for ensuring the last array method return a reference while in
+scalar context. Typically:
+
+    do_stuff(
+        books => $author->map_by("books")->to_ref,
+    );
+
+map_by is called in list context, so without ->to_ref it would have
+return an array, not an arrayref.
+
+=cut
+
+sub to_ref {
+    my $array = shift;
+    return $array;
+}
+
+=head2 @array->to_array() : @array
+
+Return the @array, regardless of context. This is mostly useful if
+called on a ArrayRef at the end of a chain of method calls.
+
+=cut
+
+sub to_array {
+    my $array = shift;
+    return @$array;
+}
+
+
+
+=head1 METHODS ON ARRAY-OF-OBJECTS/HASHES
+
+=cut
+
 *_normalized_accessor_args_subref
     = \&autobox::Transform::_normalized_accessor_args_subref;
 
@@ -372,26 +498,27 @@ sub __invoke_by {
     $args //= [];
     if ( ref($array->[0] ) eq "HASH" ) {
         ( defined($args) && (@$args) ) # defined and isn't empty
-            and Carp::croak("${invoke}_by('$accessor'): \$args ($args) only supported for method calls, not hash key access");
+            and Carp::croak("${invoke}_by([ '$accessor', \@args ]): \@args ($args) only supported for method calls, not hash key access");
         $invoke .= "_key";
     }
 
+    ###JPL: move up
     ref($args) eq "ARRAY"
-        or Carp::croak("${invoke}_by('$accessor', \$args): \$args ($args) is not an array ref");
+        or Carp::croak("${invoke}_by([ '$accessor', \@args ]): \@args ($args) is not a list");
 
     if( $subref_name ) {
         ref($subref) eq "CODE"
-            or Carp::croak("${invoke}_by('$accessor', \$args, \$$subref_name): \$$subref_name ($subref) is not an sub ref");
+            or Carp::croak("${invoke}_by([ '$accessor', \@args ], \$$subref_name): \$$subref_name ($subref) is not an sub ref");
     }
 
     my %seen;
     my $invoke_sub = {
-        map      => sub { [ CORE::map  { $_->$accessor( @$args ) } @$array ] },
-        map_key  => sub { [ CORE::map  { $_->{$accessor}         } @$array ] },
-        grep     => sub { [ CORE::grep { $subref->( local $_ = $_->$accessor( @$args ) ) } @$array ] },
-        grep_key => sub { [ CORE::grep { $subref->( local $_ = $_->{$accessor}         ) } @$array ] },
-        uniq     => sub { [ CORE::grep { ! $seen{ $_->$accessor( @$args ) // "" }++ } @$array ] },
-        uniq_key => sub { [ CORE::grep { ! $seen{ $_->{$accessor}         // "" }++ } @$array ] },
+        map        => sub { [ CORE::map  { $_->$accessor( @$args ) } @$array ] },
+        map_key    => sub { [ CORE::map  { $_->{$accessor}         } @$array ] },
+        filter     => sub { [ CORE::grep { $subref->( local $_ = $_->$accessor( @$args ) ) } @$array ] },
+        filter_key => sub { [ CORE::grep { $subref->( local $_ = $_->{$accessor}         ) } @$array ] },
+        uniq       => sub { [ CORE::grep { ! $seen{ $_->$accessor( @$args ) // "" }++ } @$array ] },
+        uniq_key   => sub { [ CORE::grep { ! $seen{ $_->{$accessor}         // "" }++ } @$array ] },
     }->{$invoke};
 
     my $result = eval { $invoke_sub->() }
@@ -440,18 +567,18 @@ sub map_by {
 
 
 
-=head2 @array->grep_by($accessor, $grep_subref = *is_true*) : @array | @$array
+=head2 @array->filter_by($accessor, $filter_subref = *is_true*) : @array | @$array
 
 $accessor is either a string, or an arrayref where the first item is a
 string.
 
 Call the $accessor on each object in the list, or get the hash key
-value on each hashref in the list. The default $grep_subref includes
+value on each hashref in the list. The default $filter_subref includes
 true values in the result @array.
 
 Examples:
 
-    my @prolific_authors = $authors->grep_by("is_prolific");
+    my @prolific_authors = $authors->filter_by("is_prolific");
 
 Alternatively the $accessor is an arrayref. The first item is the
 accessor name, and the rest of the items are passed as args the method
@@ -459,50 +586,60 @@ call. This only works when working with objects, not with hashrefs.
 
 Examples:
 
-    my @books_to_charge_for = $books->grep_by([ price_with_tax => $tax_pct ]);
+    my @books_to_charge_for = $books->filter_by([ price_with_tax => $tax_pct ]);
 
 
-=head3 The $grep_subref
+=head3 The $filter_subref
 
-The $grep_subref is called with the value returned from the $accessor
+The $filter_subref is called with the value returned from the $accessor
 to check whether this item should remain in the list (default is to
 check for true values).
 
-The $grep_subref should return a true value to remain. $_ is set to
+The $filter_subref should return a true value to remain. $_ is set to
 the current $value.
 
 Examples:
 
-    my @authors = $authors->grep_by(
+    my @authors = $authors->filter_by(
         "publisher",
         sub { $_->name =~ /Orbit/ },
     );
 
-    my @authors = $authors->grep_by(
+    my @authors = $authors->filter_by(
         [ publisher_affiliation => "with" ],
         sub { /Orbit/ },
     );
 
-Note: if you do something complicated with the $grep_subref, it might
+Note: if you do something complicated with the $filter_subref, it might
 be easier and more readable to simply use C<$array-$<gt>grep()> from
 L<autobox::Core>.
 
+
+=head3 Alias
+
+C<grep_by> is an alias for C<filter_by>. Unlike C<grep> vs C<filter>
+it works exaclty the same way.
+
 =cut
 
-sub grep_by {
+sub filter_by {
     my $array = shift;
-    my ($accessor, $args, $grep_subref) = _normalized_accessor_args_subref(@_);
-    $grep_subref //= sub { !! $_ };
-    # grep_by $value, if passed the method value must match the value?
+    my ($accessor, $args, $filter_subref) = _normalized_accessor_args_subref(@_);
+    $filter_subref //= sub { !! $_ };
+    # filter_by $value, if passed the method value must match the value?
     return __invoke_by(
-        "grep",
+        "filter",
         $array,
         $accessor,
         $args,
-        grep_subref => $grep_subref,
+        filter_subref => $filter_subref,
     );
 }
 
+*grep_by = \&filter_by;
+
+
+
 =head2 @array->uniq_by($accessor) : @array | @$array
 
 $accessor is either a string, or an arrayref where the first item is a
@@ -603,14 +740,14 @@ sub __core_group_by {
         # Hash key
         if ( ref($array->[0] ) eq "HASH" ) {
             defined($args)
-                and Carp::croak("$name('$accessor'): \$args ($args) only supported for method calls, not hash key access. Please specify an undef if needed.");
+                and Carp::croak("$name([ '$accessor', \@args ]): \@args ($args) only supported for method calls, not hash key access.");
             "key";
         }
         # Method
         else {
             $args //= [];
             ref($args) eq "ARRAY"
-                or Carp::croak("$name('$accessor', \$args, \$value_sub): \$args ($args) is not an array ref");
+                or Carp::croak("$name([ '$accessor', \@args ], \$value_sub): \@args ($args) is not a list");
             "method";
         }
     };
@@ -642,7 +779,7 @@ sub group_by {
 
     $value_sub //= sub { $_ };
     ref($value_sub) eq "CODE"
-        or Carp::croak("group_by('$accessor', [], \$value_sub): \$value_sub ($value_sub) is not a sub ref");
+        or Carp::croak("group_by([ '$accessor', \@args ], \$value_sub): \$value_sub ($value_sub) is not a sub ref");
 
     return __core_group_by("group_by", $array, $accessor, $args, $value_sub);
 }
@@ -714,77 +851,8 @@ sub group_by_array {
 }
 
 
-=head2 @array->flat() : @array | @$array
 
-Return a (one level) flattened array, assuming the array items
-themselves are array refs. I.e.
-
-    [
-        [ 1, 2, 3 ],
-        [ "a", "b" ],
-        [ [ 1, 2 ], { 3 => 4 } ]
-    ]->flat
-
-returns
-
-    [ 1, 2, 3, "a", "b ", [ 1, 2 ], { 3 => 4 } ]
-
-This is useful if e.g. a C<-E<gt>map_by("some_method")> returns
-arrayrefs of objects which you want to do further method calls
-on. Example:
-
-    # ->books returns an arrayref of Book objects with a ->title
-    $authors->map_by("books")->flat->map_by("title")
-
-Note: This is different from autobox::Core's ->flatten, which reurns a
-list rather than an array and therefore can't be used in this
-way.
-
-=cut
-
-sub flat {
-    my $array = shift;
-    ###JPL: eval and report error from correct place
-    my $result = [ map { @$_ } @$array ];
-    return wantarray ? @$result : $result;
-}
-
-=head2 @array->to_ref() : $arrayref
-
-Return the reference to the @array, regardless of context.
-
-Useful for ensuring the last array method return a reference while in
-scalar context. Typically:
-
-    do_stuff(
-        books => $author->map_by("books")->to_ref,
-    );
-
-map_by is called in list context, so without ->to_ref it would have
-return an array, not an arrayref.
-
-=cut
-
-sub to_ref {
-    my $array = shift;
-    return $array;
-}
-
-=head2 @array->to_array() : @array
-
-Return the @array, regardless of context. This is mostly useful if
-called on a ArrayRef at the end of a chain of method calls.
-
-=cut
-
-sub to_array {
-    my $array = shift;
-    return @$array;
-}
-
-
-
-=head1 AUTOBOX HASH METHODS
+=head1 METHODS ON HASH
 
 =cut
 
@@ -977,7 +1045,7 @@ sub map_each_to_array {
 
 
 
-sub grep_each {
+sub filter_each {
     my $hash = shift;
     my ($subref) = @_;
     $subref ||= sub { !! $_ }; # true?
@@ -998,18 +1066,22 @@ sub grep_each {
 
     return wantarray ? %$new_hash : $new_hash;
 }
-*grep = \&grep_each;
+{
+    no warnings "once";
+    *grep_each = \&filter_each;
+}
 
-sub grep_each_defined {
+sub filter_each_defined {
     my $hash = shift;
-    return &grep($hash, sub { defined($_) });
+    return &filter_each($hash, sub { defined($_) });
 }
 {
     no warnings "once";
-    *grep_defined = \&grep_each_defined;
+    *grep_each_defined = \&filter_each_defined;
 }
 
 
+
 =head2 %hash->to_ref() : $hashref
 
 Return the reference to the %hash, regardless of context.
@@ -1111,16 +1183,17 @@ Perl equivalent.
 
 
 
-    ### grep_by - method call: $books are Book objects
+    ### filter_by - method call: $books are Book objects
     my $sold_out_books = [ grep { $_->is_sold_out } @$books ];
+    my $sold_out_books = $books->filter_by("is_sold_out");
     my $sold_out_books = $books->grep_by("is_sold_out");
 
     my $books_in_library = [ grep { $_->is_in_library($library) } @$books ];
-    my $books_in_library = $books->grep_by([ is_in_library => $library ]);
+    my $books_in_library = $books->filter_by([ is_in_library => $library ]);
 
-    ### grep_by - hash key: $books are book hashrefs
+    ### filter_by - hash key: $books are book hashrefs
     my $sold_out_books = [ grep { $_->{is_sold_out} } @$books ];
-    my $sold_out_books = $books->grep_by("is_sold_out");
+    my $sold_out_books = $books->filter_by("is_sold_out");
 
 
 
